-------------------------------------------------------
--! @file comps4.vhd
--! @brief 4-bit signed x > y comparator
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-03
-------------------------------------------------------

entity comps4 is
  port (
    x : in bit_vector (3 downto 0);
    y : in bit_vector (3 downto 0);
    xgty : out bit
    );
end entity comps4;

architecture comps4_arch of comps4 is

  component adder4 is
      port (
    cin : in bit;
    x : in bit_vector (3 downto 0);
    y : in bit_vector (3 downto 0);
    sum : out bit_vector (3 downto 0);
    cout : out bit
    );
end component adder4;
  
signal bin,bout : bit;
signal y_l,diff : bit_vector (3 downto 0);
signal diff3, diff3_l : bit;
signal overflow : bit;

begin
  ADDER : component adder4 port map
    (bin,x,y_l,diff,bout);
    
  bin <= '1';
  y_l <= not(y);
  overflow <= (x(3) xor y(3)) and (x(3) xor diff(3));
  diff3 <= diff(3);
  diff3_l <= not(diff3);
  
  with overflow select
    xgty <=
      diff3_l when '0',
      diff3   when '1',
      'X' when others;
  
end architecture comps4_arch;
